/*===================================
		Copyright Yakibomb
*///=================================

if (!("ConstantNamingConvention" in ROOT))
{
	ROOT["ConstantNamingConvention"] <- true;
	foreach (a,b in Constants)
		foreach (k,v in b)
			if (v == null)
				ROOT[k] <- 0;
			else
				ROOT[k] <- v;
}

if (!("GetPropInt" in ROOT))
{
	foreach (k, v in NetProps.getclass())
		if (k != "IsValid")
			getroottable()[k] <- NetProps[k].bindenv(NetProps);
}

/*===================================
//		Function Checks
*///=================================

function VSCONDS_IsCondNumInvalid(condition, entity) {
	if ( type(condition) != "integer" ) return true;
	if ( !VSCONDS_IsEntValid(entity) ) return true;
	return false;
}
function VSCONDS_ClampCondValue(condition) {
	local max = VSCOND_COND_NAMES.len() - 1;
	max = max < 0 ? 0 : max;
	local min = 0;
	return condition <= min ? min : condition > max ? max : condition;
}

function VSCONDS_IsEntValid(ent) {
	if ( ent && ent.IsValid() ) return ent;
	return null
}
function VSCONDS_IsEntDead(ent) {
	
	if ( GetPropInt(ent, "m_lifeState") ) //checks if entity is not alive
		return true;
	
	return false;
}

/*===================================
//		Initializer
*///=================================

function VSCONDS_CondEntInit (ent) {
	local ent_cond_table = format("%s_%d", "VSCOND_TABLE", ent.entindex());
	if ( !( VSCOND_ENTS_UNDER_CONDS.find(ent_cond_table) ) ) {
		ROOT[ent_cond_table] <-  { };
		ROOT[ent_cond_table][VSCond_MAX_Conditions] <- ent;
		ROOT[ent_cond_table][VSCond_MAX_Conditions + 1] <- ent;
	}
		
	local finder = VSCOND_ENTS_UNDER_CONDS.find(ent_cond_table);
	if ( !finder ) ::VSCOND_ENTS_UNDER_CONDS.append(ent_cond_table);
}

function VSCONDS_GetCondFunc (condition) {
	if ( type(condition) != "integer" || condition < 0 ) condition = 0;
	
	local condition_name = format("%s%s", "VSCONDFUNC_", VSCOND_COND_NAMES[condition]);
	local funcsh = ROOT[condition_name];
//	foreach( k, v in funcsh ) ClientPrint(null, 3, k + " " + v);
	return funcsh;
}


function VSCONDS_RemoveThinkToCond(ent,condition) {
//	if ( type(condition) != "integer" ) return;
//	ent = type(ent) == "integer" ? ent : ent.entindex();
	
	local Name = format("VSConds_%d_%d_*",ent,condition);
	DoEntFire(Name.tostring(), "Trigger", "", 0.0, worldspawn, worldspawn);
}
function VSCONDS_RemoveThinkToCond_ALL(ent) {
	ent = type(ent) == "integer" ? ent : ent.entindex();
	if (ent==0) return;
	local Name = format("VSConds_%d_*",ent);
	DoEntFire(Name.tostring(), "Trigger", "", 0.0, worldspawn, worldspawn);
}

/*=======================================
//	Attachments, aka things parented to the entity with condition. Usually for particles, but can be anything
*///=====================================
function VSCONDS_AddAttachmentEnt(ent,condition,attachment) {
	ent = ent.entindex()
	
	local parent_array = format("%s_%d", "VSCOND_ATTACH_PARENT", ent)
	if ( !( parent_array in ROOT ) ) ROOT[parent_array] <- [];
	
	local child_array = format("%s_%d_%d", "VSCOND_ATTACH_CHILD", ent, condition)
	if ( !( child_array in ROOT ) ) ROOT[child_array] <- [];
	
	ROOT[child_array].append(attachment);
	ROOT[parent_array].append(ROOT[child_array]);
}
function VSCONDS_RemoveAttachmentEnt(ent,condition) {
	local child_array_name = format("%s_%d_%d", "VSCOND_ATTACH_CHILD", ent, condition);
	if ( !( child_array_name in ROOT ) ) return;
	local child_array = ROOT[child_array_name];
	
	local parent_array_name = format("%s_%d", "VSCOND_ATTACH_PARENT", ent)
	if ( !( parent_array_name in ROOT ) ) return;
	
	foreach ( key, attachment in child_array ) {
		if ( attachment && attachment.IsValid() ) attachment.Kill();
	}
	delete ROOT[child_array_name];
}
	
function VSCONDS_RemoveAttachmentEnt_ALL(ent) {
	local parent_array_name = format("%s_%d", "VSCOND_ATTACH_PARENT", ent)
	if ( !( parent_array_name in ROOT ) ) return;
	
	local parent_array = ROOT[parent_array_name];
	
	foreach ( key, child_array in parent_array )
	{
		foreach ( index, attachment in child_array ) {
			if ( attachment && attachment.IsValid() ) attachment.Kill();
		}
	}
	delete ROOT[parent_array_name];
}

	
/*=======================================
//	Particle stuff
*///=====================================

//	Quirk with this one: This stops any parented particles, but it stops all, not just one.
function VSCONDS_StopParticle(player) {
	EntFireByHandle(player, "DispatchEffect", "ParticleEffectStop", 0.0, player, player)
}

	
/*=======================================
//	Creates a team-specific particle, then parents it to an entity.
	The team that the provider (provider) is on is what the function selects as the particle.
	It is then parented to the entity (ent) based on an vector you set it to.
	
	Can optionally set RED (team2), BLU (team3), or NEUTRAL (team5), in that order.
	Defaults to RED if no other particles set.
*///=====================================
function VSCONDS_Distinguish_Team(ent,provider,team2,team3,team5) {
	local particle = null
	if ( team2 && !team3 && !team5 ) {
		particle = team2;
	}
	else if ( team2 && team3 && !team5 ) {
		particle = ent.GetTeam() > 2 ? team3 : team2;
	}
	else if ( team2 && team3 && team5 ) {
		if ( provider == null ) {
			particle = team5;
		}
		else if ( provider.GetTeam() != 2 && provider.GetTeam() != 3 ) {
			particle = team5;
		}
		else { particle = provider.GetTeam() > 2 ? team3 : team2; }
	}
	else if ( !team2 || !team3 || !team5 ) {
		if ( team2 ) {
			particle = team2;
		}
		else if ( team3 ) {
			particle = team3;
		}
		else {
			particle = team5;
		}
	}
	return particle;
}

function VSCONDS_CreateTeamParticle(condition, provider=null, ent=null, particle_origin=null, team2="ping_circle", team3=null, team5=null) {
	if ( !ent || !ent.IsValid() ) return null;
	
	local particle = VSCONDS_Distinguish_Team(ent,provider,team2,team3,team5);
	
	local parent_the_entity = true
	if ( !particle_origin ) {
		particle_origin = ent.GetOrigin();
		parent_the_entity = false;
	}
	
	particle = SpawnEntityFromTable("info_particle_system", 
	{
	//	targetname		= UniqueString(),
		effect_name		= particle,
		start_active	= 1,
		flag_as_weather	= 0,
		origin			= particle_origin,
	})
	if ( parent_the_entity ) {
		DoEntFire("!self", "SetParent", "!activator", 0, ent, particle)
	}
	
	VSCONDS_AddAttachmentEnt(ent,condition,particle);
	return particle;

}


/*=======================================
//	Creates a team-specific drip particle, then parents it to an entity.
	Same as above, but utilizes trigger_particle to create the "drip" effects. (like Jarate, afterburn, etc)
*///=====================================
function VSCONDS_CreateTeamParticleDrip(condition,provider=null, ent=null, particle_origin=Vector(0,0,0), team2="ping_circle", team3=null, team5=null) {
	if ( !ent || !ent.IsValid() ) return null;
	
	local particle = VSCONDS_Distinguish_Team(ent,provider,team2,team3,team5);
	
	particle = SpawnEntityFromTable("trigger_particle", 
	{
//		targetname		= UniqueString(),
		particle_name	= particle,
		spawnflags		= 65,	//1 + 64
		attachment_type	= 1,
		origin			= Vector(0,0,0),
		mins		= Vector(0,0,0),
		maxs		= Vector(0,0,0),
	});
	EntFireByHandle(particle, "StartTouch", null, 0, ent, ent);
	EntFireByHandle(particle, "Kill", null, 0.01, ent, ent);
	
	return particle;
}


/*=======================================
//	Creates a team-specific Beam particle, then parents it to an entity.
	Beam particle's control point 1 set to entity's center.
	The particle's origin is where the beam originates from, BUT the particle ent itself parents to the PROVIDER--not the main entity.
*///=====================================
function VSCONDS_CreateBeamParticle(condition, provider=null, ent=null, particle_origin=null, team2="ping_circle", team3=null, team5=null) {
	
	if ( !ent || !ent.IsValid() ) return null;
	
	local particle = VSCONDS_Distinguish_Team(ent,provider,team2,team3,team5);
	
	if ( !particle_origin )
		particle_origin = ent.GetOrigin()

	
	local center = ent.GetHiddenWearable();
	
	particle = SpawnEntityFromTable("info_particle_system", 
	{
//		targetname		= UniqueString(),
		effect_name		= particle,
		start_active	= 1,
		flag_as_weather	= 0,
		origin			= particle_origin,
		cpoint1			= center.GetName()
	});
	DoEntFire("!self", "SetParent", "!activator", 0, provider, particle);
	SetPropEntity(particle,"m_hControlPointEnts",center);
	
	VSCONDS_AddAttachmentEnt(ent,condition,particle);
	
	return particle;
}	


/*=======================================
//	Creates a team-specific model, then parents it to an entity.
*///=====================================
function VSCONDS_CreateTeamProp(condition, provider=null, ent=null, model_origin=null, team2="models/buildables/dispenser_light.mdl", team3=null, team5=null) {
	
	if ( !ent || !ent.IsValid() ) return null;
	
	local model = VSCONDS_Distinguish_Team(ent,provider,team2,team3,team5);
	
	local parent_the_entity = true;
	if ( !model_origin ) {
		model_origin = ent.GetOrigin();
		parent_the_entity = false;
	}
	
	if ( ent.IsPlayer() ) {
		local hidden_wearable = ent.GetHiddenWearable();
		
		model = ent.CreateConditionWearable(condition, model, model_origin);
		DoEntFire("!self", "SetParent", "!activator", 0, hidden_wearable, model);
	//	DoEntFire("!self", "SetParentAttachment", "halopoint1", 0.01, hidden_wearable, model);
		SetPropInt(model, "m_fEffects", 0);
	}
	else {
		PrecacheModel(model);
		model = SpawnEntityFromTable("prop_dynamic", 
		{
			model					= model,
			origin					= model_origin,
			DisableBoneFollowers	= true,
		});
		DoEntFire("!self", "SetParent", "!activator", 0, ent, model);
		VSCONDS_AddAttachmentEnt(ent,condition,model);
	}
	/*
	if ( parent_the_entity ) {
		DoEntFire("!self", "SetParent", "!activator", 0, ent, model);
	}*/
	
	return model;
}

/*=======================================
//	Creates a powerup particle, similar to Marked for Death and mannpower powerups
	Only has basic support for now (they overlap if more than one set)
*///=====================================
function VSCONDS_CreatePowerupIconParticle(condition, provider=null, ent=null, particle_origin=null, team2="mark_for_death", team3=null, team5=null) {
	
	if ( !ent || !ent.IsValid() ) return null;
	
	local particle = VSCONDS_Distinguish_Team(ent,provider,team2,team3,team5);
	
	local hidden_wearable = ent.GetHiddenWearable();
	local position = ent.EyePosition() + ent.EyeAngles().Up() * 16;
	
	particle = SpawnEntityFromTable("info_particle_system", 
	{
	//	targetname		= "particle_"+UniqueString(),
		effect_name		= particle,
		start_active	= 1,
		flag_as_weather	= 0,
		origin			= position,
		//cpoint1			= hidden_wearable.GetName(),
	});
	
	DoEntFire("!self", "SetParent", "!activator", 0.0, hidden_wearable, particle);


	VSCONDS_AddAttachmentEnt(ent,condition,particle);
	
	return particle;
}


/*=======================================
// Utility
*///=====================================

/*=======================================
	Collect all child entities on parent entity, puts it into an array.
	Option to collect parent as well.
*///=====================================
function VSCONDS_CollectChildOnParent(entity, alsoparent=false) {
	local child_list = [];
	if (alsoparent) child_list = [entity];
	
	for ( local item = entity.FirstMoveChild(); item != null ; item = item.NextMovePeer() )
	{
		child_list.append(item)
	}
	return child_list;
}

/*=======================================
	Colors entity, including all child entities
*///=====================================
function VSCONDS_Color(ent, r,g,b) {
	local child_list = VSCONDS_CollectChildOnParent(ent, true);
	
	foreach ( child in child_list ) {
		child.KeyValueFromString("rendercolor", format("%d %d %d", r, g, b) )
	}
}
	
	
/*=======================================
	Set Parent attachment
*///=====================================
function VSCONDS_SetParentAttachment(parent,child,attachment=null)
{
	EntFireByHandle(child,"SetParent","!activator",0.0,parent,null);
	if ( attachment )
		EntFireByHandle(child,"SetParentAttachment",attachment.tostring(),0.01,null,null);
}


/*=======================================
//	Create Invisible Wearable
	Used mainly for CreateBeamParticle(). May be unfinished.
*///=====================================

function CBaseEntity_GetHiddenWearable(modelpath=null)
{
	local wearable;
	local ent = this;
	
	if ( !( ent in VSCOND_HIDDEN_WEARABLES ) ) {
		wearable = VSCONDS_SpawnWearable(ent, modelpath, ent.GetCenter());
		wearable.SetModelScale(0.5,0);
		SetPropInt(wearable, "m_fEffects", 129);
		::VSCOND_HIDDEN_WEARABLES[ent] <- wearable;
	}
	else {
		wearable = VSCOND_HIDDEN_WEARABLES[ent];
		
		if ( modelpath ) {
			PrecacheModel(modelpath);
			local wearable_modelindex = GetModelIndex(modelpath);
			SetPropInt(wearable, "m_nModelIndex", wearable_modelindex);
		}
	}
	
	return wearable;
}
	
function VSCONDS_SpawnWearable(ent, model = "models/error.mdl", model_origin = null, initialized = true)
{
	if ( model == null ) {model = "models/player/items/all_class/all_halo.mdl"; initialized = false; }
	else if ( type(model) != "string" ) { model = "models/error.mdl"; }
	if ( model_origin == null ) model_origin = ent.GetOrigin();
	
	PrecacheModel(model);
	local wearable_model = GetModelIndex(model);
	
	local wearable = Entities.CreateByClassname("tf_wearable");
	
	SetPropInt(wearable, "m_AttributeManager.m_Item.m_iEntityLevel", 1);
//	SetPropInt(wearable, "m_bClientSideAnimation", 1);
	SetPropInt(wearable, "m_iTeamNum", ent.GetTeam());
	SetPropInt(wearable, "m_Collision.m_usSolidFlags", FSOLID_NOT_SOLID);
	SetPropInt(wearable, "m_CollisionGroup", 11);
	
	SetPropInt(wearable, "m_nModelIndex", wearable_model);
	SetPropInt(wearable, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", 65535);
	
	SetPropBool(wearable, "m_bValidatedAttachedEntity", true)
	SetPropBool(wearable, "m_AttributeManager.m_Item.m_bInitialized", initialized);
	
	SetPropEntity(wearable, "m_hOwnerEntity", ent);
	wearable.SetOwner(ent);

	Entities.DispatchSpawn(wearable);
	
	DoEntFire("!self", "SetParent", "!activator", 0.0, ent, wearable);
	wearable.SetAbsOrigin(model_origin)
	wearable.SetAbsAngles(ent.GetLocalAngles());
	
	wearable.KeyValueFromString("targetname", "tf_wearable_vscript"+UniqueString())
	
	return wearable
}

function CBaseEntity_CreateConditionWearable(condition, model = "models/error.mdl", model_origin = null)
{
	local wearable = VSCONDS_SpawnWearable(this, model, model_origin, true);
	VSCONDS_AddAttachmentEnt(this,condition,wearable);
	return wearable
}

/*=======================================
	Unfinished stuff ahead!
*///=====================================


/*

function CTFPlayer_AddInvisibleAttribute(attr="",val=1.0,dur=-1) {
	if ( this.GetClassname() != "player" ) { return }
	
	if ( !( this in InvisibleAttribute ) ) {
		::InvisibleAttribute[this] <- {}
		local wearable = this.CreateHiddenWearable()
		::InvisibleAttribute[this] <- wearable
	}
	local wearable = InvisibleAttribute[this]
	
	local scope = null
	if( this.ValidateScriptScope() )
	{
		scope = this.GetScriptScope()
		
		if ( !("Weapon_Attr" in scope ) ) scope.Weapon_Attr <- []
		scope.Weapon_Attr.append(attr)
		
		if ( !("Weapon_Val" in scope ) ) scope.Weapon_Val <- []
		scope.Weapon_Val.append(val)
	}
	
	foreach (i, v in scope.Weapon_Name)
	{
		wearable.AddAttribute(scope.Weapon_Attr[i],scope.Weapon_Val[i],-1)
		printl(i + ": " + v)
	}
	//wearable.AddAttribute(attr,val,dur)
	wearable.ReapplyProvision()
}

function CBaseEntity_RemoveInvisibleAttribute(attr="") {
	if ( !( this in InvisibleAttribute ) ) {
		return
	}
	local wearable = InvisibleAttribute[this]
	wearable.RemoveAttribute(attr)
}


::debug_vscond <- function(cond=1,duration=-1)
{
	for (local i = 1; i <= MaxClients() ; i++)
	{
		local ent_valid = EntIndexToHScript(i)
		if (ent_valid == null) continue
			if ( !NetProps.GetPropInt(ent_valid, "m_lifeState") ) {
				ClientPrint(null,3,format("%s has VSCond %d for %d second(s)",ent_valid.tostring(),cond,duration))
				ent_valid.AddVSCond(cond,duration,worldspawn)
			}
	}
}
*/